#include <cairo/cairo.h>
#include <string.h>
#include <assert.h>
#include "cairo-widget.h"

enum
{
    CW_NO_PROP,
    CW_EXTENT,
    CW_PARENT,
    CW_N_PROPERTIES
};
static GParamSpec *g_cw_prop[CW_N_PROPERTIES] = {0};

typedef struct _DiyaCairoWidgetPrivate
{
    DiyaObject * parent_object;
    diya_rect_t extent;
    DiyaCairoWidget * parent_widget;
    // TODO list of children
    gchar string[64];
} DiyaCairoWidgetPrivate;

G_DEFINE_TYPE_WITH_PRIVATE(DiyaCairoWidget, diya_cairo_widget, DIYA_TYPE_OBJECT);


static void diya_cairo_widget_dispose(GObject* object)
{
    g_debug("diya_cairo_widget_dispose");
    G_OBJECT_CLASS(diya_cairo_widget_parent_class)->dispose(object);
}

static void diya_cairo_widget_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
{
    DiyaCairoWidget * self = DIYA_CAIRO_WIDGET(object);
    DiyaCairoWidgetPrivate* priv = diya_cairo_widget_get_instance_private(self);
    diya_rect_t * rect;
    switch (property_id)
    {
    case CW_PARENT:
        priv->parent_widget = g_value_get_pointer(value);
        break;
    case CW_EXTENT:
        rect = g_value_get_pointer(value);
        assert(rect);
        memcpy(&priv->extent,rect, sizeof(priv->extent)); 
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_cairo_widget_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
{
    DiyaCairoWidget * self = DIYA_CAIRO_WIDGET(object);
    DiyaCairoWidgetPrivate* priv = diya_cairo_widget_get_instance_private(self);
    switch (property_id)
    {
    case CW_PARENT:
        g_value_set_pointer(value, priv->parent_widget);
        break;
    case CW_EXTENT:
        g_value_set_pointer(value, &priv->extent);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}


static void diya_cairo_widget_init(DiyaCairoWidget *self)
{
    DiyaCairoWidgetPrivate* priv = diya_cairo_widget_get_instance_private(self);
    memset(&priv->extent,0,sizeof(priv->extent));
    priv->parent_widget = NULL;
    priv->string[0] = '\0';
}

static const gchar *diya_cairo_widget_to_string(DiyaObject *object)
{
    DiyaCairoWidget * self = DIYA_CAIRO_WIDGET(object);
    DiyaCairoWidgetClass *class = DIYA_CAIRO_WIDGET_GET_CLASS(object);
    DiyaCairoWidgetPrivate* priv = diya_cairo_widget_get_instance_private(self);

    g_snprintf(priv->string,sizeof(priv->string),
        "%s (%dx%d) at %d@%d",
        class->name,
        priv->extent.x, priv->extent.y,
        priv->extent.width, priv->extent.height
    );
    return priv->string;
}

static void diya_cairo_widget_class_init(DiyaCairoWidgetClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    DiyaObjectClass *base_class = DIYA_OBJECT_CLASS(class);
    DIYA_CAIRO_WIDGET_SET_NAME(class, "");
    gobject_class->dispose = diya_cairo_widget_dispose;
    gobject_class->set_property = diya_cairo_widget_set_property;
    gobject_class->get_property = diya_cairo_widget_get_property;
    g_cw_prop[CW_EXTENT] = g_param_spec_pointer("extent", NULL, "Widget extent relative to its parent", G_PARAM_READWRITE); // 
    g_cw_prop[CW_PARENT] = g_param_spec_pointer("parent", NULL, "Parent widget", G_PARAM_READWRITE); // 

    base_class->to_string = diya_cairo_widget_to_string;

    g_object_class_install_properties (gobject_class, CW_N_PROPERTIES, g_cw_prop);
}
